@@ -1047,10 +1047,7 @@ authenticate_perform (Authenticate *args, GP11Arguments *base)
 		/* Protected authentication path, just go to perform */
 		if (args->protected_auth) {
 			args->state = AUTHENTICATE_PERFORM;
-			do {
-				rv = authenticate_perform (args, base);
-			} while (rv == CKR_PIN_INCORRECT);
-			return rv;
+			return authenticate_perform (args, base);
 		}
 
 		/* Get the label for a prompt */
@@ -1139,9 +1136,9 @@ authenticate_init (Authenticate *auth, GP11Slot *slot, GP11Object *object)
 	g_assert (GP11_IS_OBJECT (object));
 	
 	module = gp11_slot_get_module (slot);
-	if (gp11_module_get_auto_authenticate (module)) {
+	if (gp11_module_get_auto_authenticate (module) & GP11_AUTHENTICATE_OBJECTS) {
 		auth->state = AUTHENTICATE_CAN;
-		auth->protected_auth = _gp11_slot_is_protected_auth_path (slot);
+		auth->protected_auth = gp11_slot_has_flags (slot, CKF_PROTECTED_AUTHENTICATION_PATH);
 		auth->module = module;
 		auth->object = g_object_ref (object);
 	} else {
