@@ -21,8 +21,8 @@
    Author: Stef Walter <nielsen@memberwebs.com>
 */
 
-#ifndef GP11_H_
-#define GP11_H_
+#ifndef GP11_H
+#define GP11_H
 
 #include <glib.h>
 #include <glib-object.h>
@@ -41,6 +41,8 @@ G_BEGIN_DECLS
 
 GQuark              gp11_get_error_quark                    (void);
 
+GList*              gp11_list_ref_copy                      (GList *reflist);
+
 void                gp11_list_unref_free                    (GList *reflist);
 
 const gchar*        gp11_message_from_rv                    (CK_RV rv);
@@ -65,14 +67,25 @@ typedef struct GP11Attribute {
  * Used with var args in place of a length to denote that this type
  * of value follows.
  */
+
+#define GP11_BOOLEAN  ((gssize)-1)
+#define GP11_ULONG    ((gssize)-2)
+#define GP11_STRING   ((gssize)-3)
+#define GP11_DATE     ((gssize)-4)
+
+#define GP11_INVALID G_MAXULONG
+
 enum {
-	GP11_BOOLEAN = -1,
-	GP11_ULONG = -2,
-	GP11_STRING = -3,
-	GP11_DATE = -4
+	GP11_AUTHENTICATE_TOKENS = 2,
+	GP11_AUTHENTICATE_OBJECTS = 4
 };
 
-#define GP11_INVALID (gulong)-1
+/* Used on varargs functions that should end with GP11_INVALID */
+#ifdef NOT_YET_SUPPORTED
+#define GP11_INVALID_TERMINATED __attribute__((__sentinel__(G_MAXULONG)))
+#else
+#define GP11_INVALID_TERMINATED
+#endif
 
 void                gp11_attribute_init                     (GP11Attribute *attr,
                                                              gulong attr_type,
@@ -150,13 +163,16 @@ GType               gp11_attributes_get_boxed_type          (void) G_GNUC_CONST;
  
 GP11Attributes*     gp11_attributes_new                     (void);
 
-GP11Attributes*     gp11_attributes_new_empty               (gulong attr_type, ...);
+GP11Attributes*     gp11_attributes_new_empty               (gulong attr_type, 
+                                                             ...) GP11_INVALID_TERMINATED;
 
 GP11Attributes*     gp11_attributes_new_full                (GP11Allocator allocator);
 
-GP11Attributes*     gp11_attributes_newv                    (gulong attr_type, ...);
+GP11Attributes*     gp11_attributes_newv                    (gulong attr_type, 
+                                                             ...) GP11_INVALID_TERMINATED;
 
-GP11Attributes*     gp11_attributes_new_valist              (GP11Allocator allocator, va_list va);
+GP11Attributes*     gp11_attributes_new_valist              (GP11Allocator allocator, 
+                                                             va_list va);
 
 GP11Attribute*      gp11_attributes_at                      (GP11Attributes *attrs,
                                                              guint index);
@@ -276,6 +292,11 @@ GP11Module*           gp11_module_initialize                  (const gchar *path
                                                                gpointer reserved,
                                                                GError **err);
 
+gboolean              gp11_module_equal                       (gconstpointer module1,
+                                                               gconstpointer module2);
+
+guint                 gp11_module_hash                        (gconstpointer module);
+
 const gchar*          gp11_module_get_path                    (GP11Module *self);
 
 CK_FUNCTION_LIST_PTR  gp11_module_get_functions               (GP11Module *self);
@@ -290,15 +311,15 @@ gboolean              gp11_module_get_pool_sessions           (GP11Module *self)
 void                  gp11_module_set_pool_sessions           (GP11Module *self, 
                                                                gboolean pool_sessions);
 
-gboolean              gp11_module_get_auto_authenticate       (GP11Module *self);
+gint                  gp11_module_get_auto_authenticate       (GP11Module *self);
 
 void                  gp11_module_set_auto_authenticate       (GP11Module *self, 
-                                                               gboolean auto_authenticate);
+                                                               gint auto_authenticate);
 
 gboolean              gp11_module_enumerate_objects           (GP11Module *self,
                                                                GP11ObjectForeachFunc func,
                                                                gpointer user_data,
-                                                               ...);
+                                                               ...) GP11_INVALID_TERMINATED;
 
 gboolean              gp11_module_enumerate_objects_full      (GP11Module *self,
                                                                GP11Attributes *attrs,
@@ -387,6 +408,9 @@ typedef GArray GP11Mechanisms;
 
 #define gp11_mechanisms_free(a)         (g_array_free(a, TRUE))
 
+gboolean            gp11_mechanisms_check                   (GP11Mechanisms *mechanisms,
+                                                             ...);
+
 #define GP11_TYPE_SLOT             (gp11_slot_get_type())
 #define GP11_SLOT(obj)             (G_TYPE_CHECK_INSTANCE_CAST((obj), GP11_TYPE_SLOT, GP11Slot))
 #define GP11_SLOT_CLASS(klass)     (G_TYPE_CHECK_CLASS_CAST((klass), GP11_TYPE_SLOT, GP11Slot))
@@ -413,6 +437,11 @@ struct _GP11SlotClass {
 
 GType               gp11_slot_get_type                      (void) G_GNUC_CONST;
 
+gboolean            gp11_slot_equal                         (gconstpointer slot1,
+                                                             gconstpointer slot2);
+
+guint               gp11_slot_hash                          (gconstpointer slot);
+
 GP11Module*         gp11_slot_get_module                    (GP11Slot *self);
 
 CK_SLOT_ID          gp11_slot_get_handle                    (GP11Slot *self);
@@ -428,6 +457,9 @@ GP11Mechanisms*     gp11_slot_get_mechanisms                (GP11Slot *self);
 GP11MechanismInfo*  gp11_slot_get_mechanism_info            (GP11Slot *self,
                                                              gulong mech_type);
 
+gboolean            gp11_slot_has_flags                     (GP11Slot *self,
+                                                             gulong flags);
+
 #if UNIMPLEMENTED
 
 gboolean            gp11_slot_init_token                    (GP11Slot *self, 
@@ -628,7 +660,7 @@ gboolean            gp11_session_logout_finish              (GP11Session *self,
 
 GP11Object*         gp11_session_create_object              (GP11Session *self, 
                                                              GError **err, 
-                                                             ...); 
+                                                             ...) GP11_INVALID_TERMINATED; 
 
 GP11Object*         gp11_session_create_object_full         (GP11Session *self,
                                                              GP11Attributes *attrs,
@@ -647,7 +679,7 @@ GP11Object*         gp11_session_create_object_finish       (GP11Session *self,
 
 GList*              gp11_session_find_objects               (GP11Session *self,
                                                              GError **err,
-                                                             ...); 
+                                                             ...) GP11_INVALID_TERMINATED; 
 
 GList*              gp11_session_find_objects_full          (GP11Session *self,
                                                              GP11Attributes *attrs,
@@ -669,38 +701,38 @@ GList*              gp11_session_find_objects_finish        (GP11Session *self,
 GP11Object*         gp11_session_generate_key               (GP11Session *self,
                                                              GP11Mechanism *mechanism,
                                                              GError **err,
-                                                             ...);
+                                                             ...) GP11_INVALID_TERMINATED;
 
 void                gp11_session_generate_key_async         (GP11Session *self,
                                                              GP11Mechanism *mechanism,
                                                              GAsyncReadyCallback callback,
                                                              gpointer user_data,
-                                                             ...);
+                                                             ...) GP11_INVALID_TERMINATED;
 
 GP11Object*         gp11_session_generate_key_finish        (GP11Session *self,
                                                              GAsyncResult *result,
                                                              GError **err,
-                                                             ...);
+                                                             ...) GP11_INVALID_TERMINATED;
 
 gboolean            gp11_session_generate_key_pair          (GP11Session *self,
                                                              GP11Mechanism *mechanism,
                                                              GP11Object **public_key,
                                                              GP11Object **private_key,
                                                              GError **err,
-                                                             ...);
+                                                             ...) GP11_INVALID_TERMINATED;
 
 void                gp11_session_generate_key_pair_async    (GP11Session *self,
                                                              GP11Mechanism *mechanism,
                                                              GAsyncReadyCallback callback,
                                                              gpointer user_data,
-                                                             ...);
+                                                             ...) GP11_INVALID_TERMINATED;
 
 gboolean            gp11_session_generate_key_pair_finish   (GP11Session *self,
                                                              GAsyncResult *result,
                                                              GP11Object **public_key,
                                                              GP11Object **private_key,
                                                              GError **err,
-                                                             ...);
+                                                             ...) GP11_INVALID_TERMINATED;
 
 gboolean            gp11_session_seed_random                (GP11Session *self,
                                                              const guchar *seed,
@@ -1162,7 +1194,7 @@ GP11Object*         gp11_session_unwrap                      (GP11Session *self,
                                                               const guchar *input,
                                                               gsize n_input,
                                                               GError **err,
-                                                              ...);
+                                                              ...) GP11_INVALID_TERMINATED;
 
 GP11Object*         gp11_session_unwrap                      (GP11Session *self,
                                                               GP11Object *key,
@@ -1171,7 +1203,7 @@ GP11Object*         gp11_session_unwrap                      (GP11Session *self,
                                                               gsize n_input,
                                                               GCancellable *cancellable,
                                                               GError **err,
-                                                              ...);
+                                                              ...) GP11_INVALID_TERMINATED;
 
 void                gp11_session_unwrap_async                (GP11Session *self,
                                                               GP11Object *key,
@@ -1181,7 +1213,7 @@ void                gp11_session_unwrap_async                (GP11Session *self,
                                                               GCancellable *cancellable,
                                                               GAsyncReadyCallback callback,
                                                               gpointer user_data);
-                                                              ...);
+                                                              ...) GP11_INVALID_TERMINATED;
 
 GP11Object*         gp11_session_unwrap_finish               (GP11Session *self,
                                                               GAsyncResult *result,
@@ -1191,14 +1223,14 @@ GP11Object*         gp11_session_derive                      (GP11Session *self,
                                                               GP11Object *key,
                                                               gulong mech_type,
                                                               GError **err,
-                                                              ...);
+                                                              ...) GP11_INVALID_TERMINATED;
 
 GP11Object*         gp11_session_derive_full                 (GP11Session *self,
                                                               GP11Object *key,
                                                               GP11Mechanism *mech_args,
                                                               GCancellable *cancellable,
                                                               GError **err,
-                                                              ...);
+                                                              ...) GP11_INVALID_TERMINATED;
 
 void                gp11_session_derive_async                (GP11Session *self,
                                                               GP11Object *key,
@@ -1206,7 +1238,7 @@ void                gp11_session_derive_async                (GP11Session *self,
                                                               GCancellable *cancellable,
                                                               GAsyncReadyCallback callback,
                                                               gpointer user_data);
-                                                              ...);
+                                                              ...) GP11_INVALID_TERMINATED;
 
 GP11Object*         gp11_session_derive_finish               (GP11Session *self,
                                                               GAsyncResult *result,
@@ -1247,6 +1279,11 @@ GList*              gp11_objects_from_handle_array          (GP11Slot *slot,
                                                              CK_OBJECT_HANDLE_PTR handles,
                                                              CK_ULONG n_handles);
 
+gboolean            gp11_object_equal                       (gconstpointer slot1,
+                                                             gconstpointer slot2);
+
+guint               gp11_object_hash                        (gconstpointer slot);
+
 GP11Module*         gp11_object_get_module                  (GP11Object *self);
 
 GP11Slot*           gp11_object_get_slot                    (GP11Object *self);
@@ -1317,7 +1354,7 @@ gssize              gp11_object_get_size_finish             (GP11Object *self,
 
 gboolean            gp11_object_set                         (GP11Object *self,
                                                              GError **err,
-                                                             ...);
+                                                             ...) GP11_INVALID_TERMINATED;
 
 gboolean            gp11_object_set_full                    (GP11Object *self,
                                                              GP11Attributes *attrs,
@@ -1336,7 +1373,7 @@ gboolean            gp11_object_set_finish                  (GP11Object *self,
 
 GP11Attributes*     gp11_object_get                         (GP11Object *self,
                                                              GError **err,
-                                                             ...);
+                                                             ...) GP11_INVALID_TERMINATED;
 
 GP11Attributes*     gp11_object_get_full                    (GP11Object *self,
                                                              GP11Attributes *attrs,
@@ -1418,4 +1455,4 @@ guchar*             gp11_processor_close_finish            (GP11Processor *proce
 
 G_END_DECLS
 
-#endif /*GP11_H_*/
+#endif /*GP11_H*/
